//-------------------------------------------------------------------+
// FemData.h                                                         |
//-------------------------------------------------------------------+
// Definiton of CFemData-class. Encapsulates the File-Handling of FEM|
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 30.01.2001
#ifndef _DVA_FEM_DATA_H__
#define _DVA_FEM_DATA_H__

#include "FemDataError.h"
#include "FemElement.h"
#include "FemEdge.h"
#include "FemConstants.h"       // FP_TYPE
#include "FemCoord.h"
#include "FemCalc.h"
#include "ScrewDriver.h"

#include <vector>
typedef vector<tagCoord> COORDVEC;

/*!
    \brief Klasse zum Zugriff auf FEM-Daten
    \author Tobias Lang
    \version 0.8
    \date    2001
*/
class CFemData
{
private:
    int         m_iFEMVersion;      //!< Version der eingelesenen FEM-Daten
    string      m_sFEMOutFile;      //!< Name der FEM-Out-Datei
    string      m_sFEMInFile;       //!< Dateiname der FEM-In-Datei (wird nur fr Zusatzinformationen bei FEM-Out verwendet)

    int         m_iNodeCount;       //!< Anzahl der Knoten
    COORDVEC    m_vNodeCoord;       //!< Vector mit (x,y)-Koordinaten der Knotenpunkte

    CFemElement m_TriElement;       //!< Dreieckselemente
    CFemElement m_RecElement;       //!< Parallelogramm- oder Viereckselemente

    int         m_iEdgeCount;       //!< Anzahl der R&auml;nder
    CFemEdge*   m_pEdge;            //!< Array mit den R&auml;ndern

    int         m_iCalcCount;                   //!< Anzahl der RECHNEN-Kommandos (in der Regel min. 1 und max. 2)
    CFemCalc    m_pCalculation[g_iMAX_CALC];    //!< Ordnet die Randbedingungen den richtigen RECHNEN-Kommandos zu.
                                                //!< In der Regel liegen nicht mehr als zwei RECHNEN-Kommandos vor, kann aber bei Bedarf auf ein dyn. Array erweitert werden.

    FP_TYPE    *m_pUVValue[g_iMAX_CALC];        //!< Array mit den U/V-Werten pro RECHNEN-Kommando, die 2. Dimension entspricht der Anzahl der Knotenpunkte
    bool        m_bOutDataAvailable;            //!< Flag zeigt an, ob extern geladene Ergebnisdaten vorliegen.

public:
    CScrewDriver m_ScrewDriver;


public:
    //! Defaultkonstruktor
    CFemData();
    //! Destruktor
    ~CFemData();

    //! ffnet eine Textdatei im FEM-Format und wertet deren Inhalt aus
    //! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
    //! \param in_bTryToLoadFemOut Flag, ob versucht werden soll eine dazugehrige FEM-Out-Datei automatisch zu laden
    //! \param out_pEW Vektor mit den Eigenwerten von der erweiterten FEM-Out-Datei
    //! \param out_pEV Matrix mit den dazugehrigen Eigenvektoren (spaltenweise orientiert).
    void LoadFemIn(const char* in_pFileName, const bool in_bTryToLoadFemOut=false, CVectorNArray<FP_TYPE>* out_pEW=NULL, CMatrixNMArray<FP_TYPE>* out_pEV=NULL);

    //! Speichert die Daten in einer Textdatei im FEM-Format ab
    //! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
    void SaveFemIn(const char* in_pFileName);

    //! ffnet eine Textdatei im FEM-OUT-Format. Wirft bei einem Fehler eine Exception vom Typ CFemDataError.
    //! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
    //! \param out_pEW Vektor mit den Eigenwerten von der erweiterten FEM-Out-Datei
    //! \param out_pEV Matrix mit den dazugehrigen Eigenvektoren (spaltenweise orientiert).
    void LoadFemOut(const char* in_pFileName, CVectorNArray<FP_TYPE>* out_pEW=NULL, CMatrixNMArray<FP_TYPE>* out_pEV=NULL);

    //! Speichert die UV-Werte im FEM-Format ab. Werden EW und EV bergeben, werden diese abgespeichert.
    //! \param in_pFileName Dateiname mit relativen oder absoluten Pfad
    //! \param in_pEW Vektor mit den Eigenwerten von der erweiterten FEM-Out-Datei
    //! \param in_pEV Matrix mit den dazugehrigen Eigenvektoren (spaltenweise orientiert).
    void SaveFemOut(const char* in_pFileName, CVectorNArray<FP_TYPE>* in_pEW=NULL, CMatrixNMArray<FP_TYPE>* in_pEV=NULL) const;

    //! Flag zeigt an, ob extern geladene Ergebnisdaten vorliegen. D.h. eine FEM-Out-Datei wurde geladen.
    //! \return true extern geladene Ergebnisdaten liegen vor.
    bool ExternOutDataAvailable() const { return m_bOutDataAvailable; }

    //! Gibt alle Resourcen wieder frei und setzt die Daten auf Null zur&uuml;ck
    void FreeData();

    //! Zur Abfrage der Versionsnummer von der FEM-Datei
    //! \return Versionsnummer
    int GetFEMVersion() const { return m_iFEMVersion; }

    //! Legt die FEM-Version der Daten fest
    //! \param in_iFEMVersion FEM-Dateiversion
    void SetFEMVersion(const int in_iFEMVersion) { m_iFEMVersion=in_iFEMVersion; }

    //! Liefert den Namen der FEMOut-Datei laut Angabe der FEMIN-Datei.
    //! \return Dateiname oder Leerstring falls keiner vorhanden ist.
    const char* GetFEMOutFileName() { return m_sFEMOutFile.c_str(); }

    //! Setzt den Namen der FEMOut-Datei.
    //! \param Dateiname
    void SetFEMOutFileName(const char* in_pFileName) { assert(in_pFileName != NULL); m_sFEMOutFile=in_pFileName; }

    //! Liefert die Anzahl der Knotenpunkte vom FEM-Gebiet
    //! \return Knotenpunkte
    int GetNodeCount() const { return m_iNodeCount; }

    //! Reserviert fr die Knotenpunkte vom FEM-Gebiet fr n Punkte Speicher. Die alten Knotenpunkte bleiben erhalten.
    //! \param in_iReserveNodes Anzahl der zu reservierenden Knotenpunkte. Muss gr&ouml;sser als NodeCount sein.
    //! \return true bei Erfolg, false falls Speicherallokation fehl schlug
    bool ReserveNodeCount(const int in_iReserveNodes);

    //! F&uuml;gt einen neuen Knoten am Ende der Liste hinzu.
    //! \param in_iNodeNr Nullindizierte Nummer des neuen Knotens. Wird zum &Uuml;berpr&uuml;fen benutzt.
    //! \param in_fXPos x-Koordinate des Knotenpunkts
    //! \param in_fYPos y-Koordinate des Knotenpunkts
    //! \return true bei Erfolg, false falls in_iNodeNr != m_iNodeCount sein sollte
    bool BackInsertNode(const int in_iNodeNr, const float in_fXPos, const float in_fYPos);

    //! Liefert zu einem bestimmten Knotenpunkt dessen (x,y)-Koordinaten
    //! \param in_iNodeNr Nullindizierte Knotenpunktnummer, Wertebereich [0 bis GetCoordCount()-1]
    //! \param out_fXCoord X-Komponente
    //! \param out_fYCoord Y-Komponente
    //! \return true bei Erfolg, false falls die Knotenpunktnummer ausserhalb des Wertebereichs liegt, oder keine Knoten vorhanden sind
    bool GetNodeCoord(const int in_iNodeNr, float &out_fXCoord, float &out_fYCoord);

    //! Zugriff auf die Dreieckselemente des Gebiets
    //! \return Dreickselemente
    CFemElement& GetTriangleElements ()  { return m_TriElement; }

    //! Zugriff auf die Viereckselemente des Gebiets
    //! \return Viereckselemente
    CFemElement& GetRectangleElements()  { return m_RecElement; }

    //! Liefert die Anzahl der R&auml;nder
    //! \return Anzahl der R&auml;nder
    int GetEdgeCount() const { return m_iEdgeCount; }

    //! Liefert einen Rand des FEM-Gebiets
    //! \param in_iEdgeNr Nullindizierter Rand-Index, Wertebereich [0 bis GetEdgeCount()-1]
    //! \return CFemEdge Rand
    CFemEdge& GetEdge(const int in_iEdgeNr) const { return m_pEdge[in_iEdgeNr]; }

    //! Liefert die Anzahl der Berechnungen, die auf das FEM-Gebiet durchgef&uuml;hrt werden sollen
    //! \return Anzahl der RECHNEN-Kommandos
    int GetCalcCount() const { return m_iCalcCount; }

    void SetCalcCount( const int inCount ) { m_iCalcCount=inCount; }

    //! Zugriff auf die RECHNEN-Struktur
    //! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
    //! \return CFemCalc-Hilfsklasse
    const CFemCalc& GetCalc(const int in_iCalcNr) const { return m_pCalculation[in_iCalcNr]; }

    //! Weist einem Knotenpunkt das Potential (U- oder V-Wert) zu
    //! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
    //! \param in_iNodeNr Nullindizierte Knotenpunktnummer, Wertebereich [0 bis GetCoordCount()-1]
    //! \param in_fUVValue Potential welches auf dem Knoten herrscht
    void SetUVValue(const int in_iCalcNr, const int in_iNodeNr, const FP_TYPE in_fUVValue);

    //! Liefert zu einem Knotenpunkt dessen Potential (U- oder V-Wert)
    //! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
    //! \param in_iNodeNr Nullindizierte Knotenpunktnummer, Wertebereich [0 bis GetCoordCount()-1]
    //! \return Potential welches auf dem Knoten herrscht
    FP_TYPE GetUVValue(const in_iCalcNr, const int in_iNodeNr) const;

    //! Sobald die Anzahl der Knotenpunkte zum Beispiel durch eine Gebietsverfeinerung ver&auml;ndert wird
    //! muss danach das Array mit den Ergebniswerten angepasst werden. ResizeUVCount passt das Array an und setzt die Werte auf Null.
    //! \return true bei Erfolg, false sonst
    bool ResizeUVCount();

    //! Berechnet zu einem beliebigen Xi und Eta, die x,y-Koordinaten und das Potential in diesem Punkt.
	//! \param in_fUVArray
    //! \param in_iElement Nullindizierte Element-Nr. [0 bis CFemElement::GetCount()-1]
    //! \param in_fXi X-Position im Einheitselement [0 bis 1.0]
    //! \param in_fEta Y-Position im Einheitselement [0 bis 1.0]
    //! \param out_fXValue Errechneter X-Wert im FEM-Gebiet G
    //! \param out_fYValue Errechneter Y-Wert im FEM-Gebiet G
    //! \param out_fUVValue Potential, dass im errechneten Punkt herscht.
    //! \return true falls die UV-Werte in den St&uuml;tzpunkten vorliegen, ansonsten false
    bool UVInterpolationTriangle( FP_TYPE *in_fUVArray, const int in_iElement, const float in_fXi, const float in_fEta,
                         float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue);

    //! Berechnet zu einem beliebigen Xi und Eta, die x,y-Koordinaten und das Potential in diesem Punkt.
	//! \param in_fUVArray
    //! \param in_iElement Nullindizierte Element-Nr. [0 bis CFemElement::GetCount()-1]
    //! \param in_fXi X-Position im Einheitselement [0 bis 1.0]
    //! \param in_fEta Y-Position im Einheitselement [0 bis 1.0]
    //! \param out_fXValue Errechneter X-Wert im FEM-Gebiet G
    //! \param out_fYValue Errechneter Y-Wert im FEM-Gebiet G
    //! \param out_fUVValue Potential, dass im errechneten Punkt herscht.
    //! \return true falls die UV-Werte in den St&uuml;tzpunkten vorliegen, ansonsten false
    bool UVInterpolationRectangle( FP_TYPE *in_fUVArray, const int in_iElement, const float in_fXi, const float in_fEta,
                         float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue);

    //! Pr&uuml;ft ob eine Kante gegeben durch Start- und Endpunkt auf einem der R&auml;nder liegt.
    //! \param in_iStartNr Knotenpunktnummer (nicht Nullindiziert!) des Startpunktes.
    //! \param in_iEndNr Knotenpunktnummer (nicht Nullindiziert!) des Endpunktes.
    //! \param out_iEdgeNr Randnummer auf der die Kante liegt.
    //! \param out_iStartIndex Index der Knotenpunktnummer, die innerhalb der CFemEdge-Klasse benutzt wird.
    //! \param out_iEndIndex Index der Knotenpunktnummer, die innerhalb der CFemEdge-Klasse benutzt wird.
    //! \return true falls die Kante auf einem Rand liegt, sonst false.
    bool PartOfEdge(const int in_iStartNr, const int in_iEndNr, int& out_iEdgeNr, int& out_iStartIndex, int& out_iEndIndex) const;

	//! liefert das Minimum und Maximum der U oder V Werte (bei EW-Berechnungen nutzlos)
    //! \param in_iCalcNr Nullindizierter Index der Berechnung, Wertebereich [0 bis GetCalcCount()-1]
	//! \param outMin Minimum
	//! \param outMin Maximum
	void GetMinMaxValues( const in_iCalcNr, FP_TYPE &outMin, FP_TYPE &outMax ); 

private:
    //! Berechnet zu einem beliebigen Xi und Eta, die x,y-Koordinaten und das Potential in diesem Punkt.
    //! \param in_pOrgUVValue Potential oder Stromwert
    //! \param in_FemElement FemElement-Klasse mit den Elementen
    //! \param in_iElement Nullindizierte Element-Nr. [0 bis CFemElement::GetCount()-1]
    //! \param in_fXi X-Position im Einheitselement [0 bis 1.0]
    //! \param in_fEta Y-Position im Einheitselement [0 bis 1.0]
    //! \param out_fXValue Errechneter X-Wert im FEM-Gebiet G
    //! \param out_fYValue Errechneter Y-Wert im FEM-Gebiet G
    //! \param out_fUVValue Potential, dass im errechneten Punkt herscht.
    //! \return true falls die UV-Werte in den St&uuml;tzpunkten vorliegen, ansonsten false
    bool UVInterpolation(const FP_TYPE* in_pOrgUVValue, const CFemElement& in_FemElement, const int in_iElement, const float in_fXi, const float in_fEta,
                         float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue);

    //! Hilfsroutine fr FEM3-Dateiformat.
    void ResortFEM3Coord(const tagFEM3Coord* in_pFEM3Coord);

    //! Hilfsroutine zum Einlesen der FEM6-Daten
    void ReadFEM6Data(class CTokenParser& in_rParser);
};

#endif //_DVA_FEM_DATA_H__