//-------------------------------------------------------------------+
// FemEdge.h                                                         |
//-------------------------------------------------------------------+
// Definiton of CFemEdge. Represents edges from FEM-Areas.           |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 30.01.2001
#ifndef _DVA_FEM_EDGE_H__
#define _DVA_FEM_EDGE_H__

#include <assert.h>

#include <vector>
typedef vector<int> IntVEC;


/*!
    \brief Klasse zur Repr&auml;sentation von R&auml;ndern
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
class CFemEdge
{
private:
    int     m_iNodeCount;   //!< Anzahl der Knoten auf dem Rand
    IntVEC  m_vNodeNr;      //!< Array der Knotenpunkte auf dem Rand
    float   m_fCondition;   //!< Randbedingung

public:
    //! Defaultkonstruktor
    CFemEdge():
        m_iNodeCount(0),
        m_fCondition(-1)
    {}

    //! Elementkonstruktor, Allokiert das Array mit den Knoten
    //! \param in_iNodeCount Anzahl der Knotenpunkte auf dem Rand
    //! \param in_fCondition Randbedingung
    CFemEdge(const int in_iNodeCount, float in_fCondition):
        m_iNodeCount(in_iNodeCount)
    {
        m_vNodeNr.reserve(m_iNodeCount);
    }

    //! Legt die Anzahl der Knotenpunkte auf dem Rand fest, und allokiert ein dementsprechend grossen Array
    //! \param in_iNodeCount Anzahl der Knotenpunkte auf dem Rand
    //! \return true bei Erfolg, sonst false
    bool SetNodeCount(const int in_iNodeCount)
    {
        m_iNodeCount=in_iNodeCount;
        m_vNodeNr.reserve(m_iNodeCount);

        return (m_vNodeNr.capacity()==m_iNodeCount);
    }

    //! Liefert die Anzahl der Knotenpunkte auf dem Rand zur&uuml;ck
    //! \return Anzahl der Knotenpunkte
    int GetNodeCount() const { return m_iNodeCount; }

    //! Setzt die Randbedingung auf fest
    //! \param Randbedingung
    void SetCondition(const float in_fCondition) { m_fCondition=in_fCondition; }

    //! Liefert die Randbedingung zur&uuml;ck
    //! \return Randbedingung
    float GetCondition() { return m_fCondition; }

    //! Setzt den Knotenpunkt fest, der auf dem Rand liegt
    //! \param in_iIndex Nullindizierter Index des Knotens. Wertebereich: [0 bis GetCoordCount()-1]
    //! \param in_iNodeNr Knotennummer
    //! \return true bei Erfolg, sonst false
    bool SetNodeNr(const int in_iIndex, const int in_iNodeNr)
    {
        if ((in_iIndex >= 0) && (in_iIndex < m_iNodeCount))
        {
            m_vNodeNr[in_iIndex]=in_iNodeNr;
            return true;
        }
        return false;
    }

    //! F&uuml;gt von hinten einen neue Knotenpunktsnummer ein.
    //! \param in_iNodeNr Knotennummer
    void BackInsert(const int in_iNodeNr)
    {
        m_vNodeNr.push_back(in_iNodeNr);
    }

    //! Liefert den Knotenpunkt der auf dem Rand liegt
    //! \param in_iIndex Nullindizierter Index des Knotens. Wertebereich: [0 bis GetCoordCount()-1]
    //! \return Knotenpunktnummer, -1 bei einem Fehler
    int GetNodeNr(const int in_iIndex)
    {
        if ((in_iIndex >= 0) && (in_iIndex < m_iNodeCount))
            return m_vNodeNr[in_iIndex];

        return -1;
    }

    //! F&uuml;gt einem Rand hinter dem angegeben Index eine neue Knotenpunktnummer hinzu.
    //! \param in_iIndex Nullindizierter Index des Knotens, hinter dem der neue Knoten eingef&uuml;gt werden soll.
    //! \param in_iNodeNr Knotenpunktnummer.
    void Insert(const int in_iIndex, const int in_iNodeNr)
    {
        m_vNodeNr.insert(m_vNodeNr.begin()+in_iIndex, in_iNodeNr);
        m_iNodeCount++;
    }
};

#endif //_DVA_FEM_EDGE_H__