//-------------------------------------------------------------------+
// FemElement.h                                                      |
//-------------------------------------------------------------------+
// Definiton of CFemElement. Represents the elements of a FEM-Field  |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 01.02.2001
#ifndef _DVA_FEM_ELEMENT_H__
#define _DVA_FEM_ELEMENT_H__

#include "FemConstants.h"   //FP_TYPE
#include "FemCoord.h"       //tagCoord
#include "Algorithm.h"

#include "MatrixNMArray.h"	// CMatrixNMArray

#include <math.h>
#include <vector>
typedef vector<int*> NodeNrVEC;


/*!
    \brief Klasse zur Repr&auml;sentation von Finiten Elementen
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
class CFemElement
{
public:
    /*! \enum eElementType
     *   Aufz&auml;hlungstyp zur Beschreibung des Elements.<br>
     *   Vom Typ h&auml;ngt die Anzahl der dazugeh&ouml;rigen Knotenpunkte ab.
     */
    enum eElementType
    {
        eET_UNKNOWN         =0,     //!< unbekannter Typ zu Initialzwecken
        eET_TRIANGLE        =3,     //!< Dreiecke, FEM-Version 2
        eET_TRIANGLE2       =6,     //!< Dreiecke mit quadratischen Ansatz, FEM-Version 3
        eET_PARALLELOGRAMM  =4,     //!< Parallelogramm, FEM-Version 2
        eET_PARALLELOGRAMM2 =8      //!< Parallelogramm mit quadratischen Ansatz, FEM-Version 3
    };

private:
    eElementType    m_eType;    //!< Elementtyp
    int             m_iCount;   //!< Anzahl der Elemente
    NodeNrVEC       m_vNodeNr;  //!< dazugeh&ouml;rige Knotenpunkte der Elemente im Gegenuhrzeigersinn, je nach Typ mit Zwischenpunkten

    int             m_iEdgeCount;   //!< Anzahl der Kanten pro Element

public:
    //! Defaultkonstruktor
    CFemElement():
        m_eType(eET_UNKNOWN),
        m_iCount(0),
        m_iEdgeCount(0)
    {}

    //! Elementtyp-Konstruktor
    //! \param in_eType Typ des zu erstellenden Elements
    //! \param in_iElements Anzahl der Elemente
    CFemElement(const eElementType in_eType, const int in_iElements):
        m_eType(in_eType),
        m_iCount(in_iElements)
    {
        m_vNodeNr.reserve(m_iCount);

        for (int i=0; i<m_iCount; i++)
        {
            //-- Speicher fr die Knotenpunkte pro Element allokieren
            //
            int* pNodeNr=new int[m_eType];
            assert(pNodeNr != NULL);

            //-- Element der Liste hinzufgen
            //
            m_vNodeNr.push_back(pNodeNr);
        }

        SetHelperAttributes();
    }

    //! Destruktor gibt den Speicher f&uuml;r die Knotenpunkt-Liste wieder frei
    ~CFemElement()
    {
        int* temp=NULL;
        for (int i=0; i<m_vNodeNr.size(); i++)
            delete [] m_vNodeNr[i];
    }

    //! Initialisert die Elemente.<br>
    //! Intern werden SetType(..) und SetCount(..) in dieser Reihenfolge aufgerufen! Von daher sollte 
    //! der Benutzer zum Einrichten der Klasse immer Init(..) verwenden und auf die Set-Methoden verzichten, da 
    //! die Reichenfolge u. U. eine Rolle spielt. Ist der Typ vom Element gleich eET_UNKNOWN wird n&auml;mlich 
    //! kein Speicher allokiert.
    //! \param in_eType Typ des Elements darf nicht eET_UNKNOWN sein, sonst wird kein Speicher allokiert
    //! \param in_iElements Anzahl der Elemente
    //! \return true bei Erfolg, sonst false
    bool Init(const eElementType in_eType, const int in_iElements)
    {
        SetType(in_eType);
        return SetCount(in_iElements);
    }

    //! Legt den Typ des Elements fest und allokiert ein entsprechend dimensoniertes<br>
    //! Knotenpunkt-Array.
    //! \param in_eType Typ des Elements. Wird eET_UNKNOWN &uuml;bergeben wird ein allokiertes Array freigegeben.
    void SetType(const eElementType in_eType)
    {
        m_eType=in_eType;

        if (m_eType == eET_UNKNOWN)
        {
            for (int i=0; i<m_vNodeNr.size(); i++)
                delete [] m_vNodeNr[i];

            m_vNodeNr.clear();
			m_iCount=0;
        }
        SetHelperAttributes();
    }

    //! Liefert den Typ des Elements
    //! \return Elementtyp
    eElementType GetType() const { return m_eType; }

    //! Legt die Anzahl der Elemente fest und allokiert entsprechend viele Elemente. Vorhanden Elemente werden &uuml;berschrieben.
    //! \param in_iElements Anzahl der Elemente (kann auch Null sein)
    //! \return true bei Erfolg, sonst false
    bool SetCount(const int in_iElements)
    {
        //-- Vorhandenen Speicher gegebenfalls freigeben
        //
        for (int i=0; i<m_vNodeNr.size(); i++)
            delete [] m_vNodeNr[i];

        m_vNodeNr.clear();

        m_iCount=in_iElements;
        if (m_iCount == 0)
            return true;

        m_vNodeNr.reserve(m_iCount);

        if (m_eType == eET_UNKNOWN)
            return false;

        //-- Speicher neu allokieren
        //
        for (i=0; i<m_iCount; i++)
        {
            int* pNodeNr=new int[m_eType];
            assert(pNodeNr != NULL);

            m_vNodeNr.push_back(pNodeNr);
        }
        return true;
    }

    //! Liefert die Anzahl der vorhandenen Elemente
    //! \return Anzahl der Elemente
    int GetCount() const { return m_iCount; }

    //! Weist dem Element eine Knotenpunktnummer zu
    //! \param in_iElement Nullindizierte Element-Nummer [0 bis GetElements()-1]
    //! \param in_iNodeIndex Nullindizierter Arrayindex fr die Knoten [0 bis GetType()-1]
    //! \param in_iNodeNr Knotenpunkt-Nummer, der zu dem Element geh&ouml;rt
    //! \param mod_pFEM3Coord Array mit den u. U. unvollstndigen Knotenpunkten. Fehlende Knotenpunkte werden durch Interpolation erzeugt.
    //! \return true bei Erfolg, false sonst (wenn z. B. der Index ausserhalb des G&uuml;ltigkeitsbereiches lag)
    bool SetNodeNr(const int in_iElement, const int in_iNodeIndex, const int in_iNodeNr, tagFEM3Coord* mod_pFEM3Coord=NULL)
    {
        if ((in_iElement >=0) && (in_iElement < m_iCount))
        if ((in_iNodeIndex >=0) && (in_iNodeIndex < m_eType))
        {
            m_vNodeNr[in_iElement][in_iNodeIndex]=in_iNodeNr;

            if (mod_pFEM3Coord)
            {
                //-- berprfe ob die Koordinaten des Knotenpunktes vorliegen
                //
                if (mod_pFEM3Coord[in_iNodeNr-1].iNodeNr==0)
                {
                    //-- nein => Koordinaten mssen ermittelt werden
                    //
                    int iStartNodeNr=in_iNodeIndex-m_iEdgeCount;
                    int iEndNodeNr  =iStartNodeNr+1;
                    if (iEndNodeNr==m_iEdgeCount)
                        iEndNodeNr=0;

                    //-- zunchst einmal die Knotenpunkt-Nummer zuweisen
                    //
                    mod_pFEM3Coord[in_iNodeNr-1].iNodeNr=in_iNodeNr;

                    //-- Koordinaten des neuen Zwischenpunktes bestimmen
                    //
                    int iElStart=m_vNodeNr[in_iElement][iStartNodeNr]-1;
                    int iElEnd  =m_vNodeNr[in_iElement][iEndNodeNr  ]-1;

                    float xDiff=fabs(mod_pFEM3Coord[iElEnd].x-mod_pFEM3Coord[iElStart].x);
                    float yDiff=fabs(mod_pFEM3Coord[iElEnd].y-mod_pFEM3Coord[iElStart].y);

                    float nx=alMIN(mod_pFEM3Coord[iElStart].x, mod_pFEM3Coord[iElEnd].x)+xDiff/2;
                    float ny=alMIN(mod_pFEM3Coord[iElStart].y, mod_pFEM3Coord[iElEnd].y)+yDiff/2;

                    mod_pFEM3Coord[in_iNodeNr-1].x=nx;
                    mod_pFEM3Coord[in_iNodeNr-1].y=ny;
                }
            }
            return true;
        }
        return false;
    }

    //! Liefert eine Knotenpunktnummer des Elements
    //! \param in_iElement Nullindizierte Elementnummer [0 bis GetElements()-1]
    //! \param in_iNodeIndex Nullindizierter Arrayindex fr die Knoten [0 bis GetType()-1]
    //! \return Knotenpunkt-Nr, -1 falls ein Index ausserhalb des G&uuml;ltigkeitsbereiches lag<br>
    //! oder das Element vom Typ eET_UNKNOWN ist
    int GetNodeNr(const int in_iElement, const int in_iNodeIndex) const
    {
        if ((in_iElement   >= 0) && (in_iElement   < m_iCount))
        if ((in_iNodeIndex >= 0) && (in_iNodeIndex < m_eType ))
        {
            return m_vNodeNr[in_iElement][in_iNodeIndex];
        }
        return -1;
    }

    //! Liefert die Anzahl der Kanten, die ein Element umfasst. Achtung: Hat nichts mit Randbedingungen zu tun.
    //! \param in_iElement Nullindizierter Arrayindex des Elements (von 0 bis GetType()-1)
    //! \return Anzahl der Kanten
    int GetEdgeCount() const { return m_iEdgeCount; }

    //! Liefert den Start- und Endpunkt einer Kante fr ein Element.
    //! \param in_iElement Nullindizierte Elementnummer [0 bis GetElements()-1]
    //! \param in_iEdgeIndex Nullindizierte Kantennummer [0 bis GetEdgeCount()-1]
    //! \param out_iStartNodeNr Knotenpunktnummer des Startpunkts der Kante
    //! \param out_iEndNodeNr Knotenpunktnummer des Endpunktes auf der Kante
    //! \return true bei Erfolg, false, falls einer der Index-Nummern ausserhalb der Gltigkeitsbereiche lag
    bool GetEdge(const int in_iElement, const int in_iEdgeIndex, int& out_iStartNodeNr, int& out_iEndNodeNr) const
    {
        if ((in_iElement   >= 0) && (in_iElement   < m_iCount    ))
        if ((in_iEdgeIndex >= 0) && (in_iEdgeIndex < m_iEdgeCount))
        {
            //-- Der Knotenpunktindex fr den Startpunkt entspricht dem nullindizierten Kantenindex
            //
            out_iStartNodeNr=m_vNodeNr[in_iElement][in_iEdgeIndex];

            //-- Der Knotenpunktindex fr den Endpunkt ist entweder 0, falls der Startpunkt am Ende des Arrays sich befindet.
            //   Ansonsten ist der Endpunkt das darauf folgende Element im Array, somit +1.
            //
            if (in_iEdgeIndex == (m_iEdgeCount-1))
                out_iEndNodeNr=m_vNodeNr[in_iElement][0];
            else
                out_iEndNodeNr=m_vNodeNr[in_iElement][in_iEdgeIndex+1];

            return true;
        }
        return false;
    };

    //! Liefert den Start- und Endpunkt einer Kante fr ein Element unter Bercksichtung von Zwischenpunkten.
    //! \param in_iElement Nullindizierte Elementnummer [0 bis GetElements()-1]
    //! \param in_iEdgeIndex Nullindizierte Kantennummer [0 bis GetEdgeCount()-1]
    //! \param out_iStartNodeNr Knotenpunktnummer des Startpunkts der Kante
    //! \param out_iEndNodeNr Knotenpunktnummer des Endpunktes auf der Kante
    //! \return true bei Erfolg, false, falls einer der Index-Nummern ausserhalb der Gltigkeitsbereiche lag
    bool GetEdgeFEM3(const int in_iElement, const int in_iEdgeIndex, int& out_iStartNodeNr, int& out_iEndNodeNr) const
    {
        if ((in_iElement   >= 0) && (in_iElement   < m_iCount   ))
        if ((in_iEdgeIndex >= 0) && (in_iEdgeIndex < m_eType    ))
        {
            //-- Z. B. bei 0 und 1 fr in_iEdgeIndex => 0 ist der reale Index auch fr 1 usw.
            //
            int iRealStartIndex=in_iEdgeIndex/2;

            //-- Der Knotenpunktindex fr den Startpunkt entspricht dem nullindizierten Kantenindex
            //
            out_iStartNodeNr=m_vNodeNr[in_iElement][iRealStartIndex];

            int iRealEndIndex=iRealStartIndex + static_cast<int>(m_eType)/2;
            if (in_iEdgeIndex%2)
            {
                iRealEndIndex--;
                if (iRealEndIndex == (static_cast<int>(m_eType)/2 -1))
                    iRealEndIndex=static_cast<int>(m_eType)-1;
            }

            out_iEndNodeNr=m_vNodeNr[in_iElement][iRealEndIndex];
            return true;
        }
        return false;
    };

    //! Reserviert fr die Elemente vom FEM-Gebiet fr n Punkte Speicher. Die alten Elemente bleiben erhalten.
    //! \param in_iReserveElements Anzahl der zu reservierenden Elemente. Muss gr&ouml;sser als Count sein.
    //! \return true bei Erfolg, false falls Speicherallokation fehl schlug
    bool ReserveElements(const int in_iReserveElements)
    {
        if (in_iReserveElements <= m_iCount)
            return false;

        m_vNodeNr.reserve(in_iReserveElements);
        return true;
    }




    void UVInterpolation(const float in_fXi, const float in_fEta,
                         const tagCoord *in_vXYCoord, const FP_TYPE *in_fAlpha,
                         float &out_fXValue, float &out_fYValue, FP_TYPE &out_fUVValue) const
    {
        switch (m_eType)
        {
        case eET_TRIANGLE:
            {
                //-- Koordinaten-Transformation fr geradliniges Dreieck
                //
                out_fXValue=in_vXYCoord[0].x +
                           (in_vXYCoord[1].x - in_vXYCoord[0].x)*in_fXi +
                           (in_vXYCoord[2].x - in_vXYCoord[0].x)*in_fEta;
                out_fYValue=in_vXYCoord[0].y +
                           (in_vXYCoord[1].y - in_vXYCoord[0].y)*in_fXi +
                           (in_vXYCoord[2].y - in_vXYCoord[0].y)*in_fEta;

                //-- linearer Ansatz im Dreieck
                //
				out_fUVValue=in_fAlpha[0] +
                           (in_fAlpha[1] - in_fAlpha[0])*in_fXi +
                           (in_fAlpha[2] - in_fAlpha[0])*in_fEta;

                break;
            }
        case eET_PARALLELOGRAMM:
            {
                //-- Koordinaten-Transformation fr geradliniges Parallelogramm
                //
                out_fXValue=in_vXYCoord[0].x +
                           (in_vXYCoord[1].x - in_vXYCoord[0].x)*in_fXi +
                           (in_vXYCoord[3].x - in_vXYCoord[0].x)*in_fEta;
                out_fYValue=in_vXYCoord[0].y +
                           (in_vXYCoord[1].y - in_vXYCoord[0].y)*in_fXi +
                           (in_vXYCoord[3].y - in_vXYCoord[0].y)*in_fEta;

                //-- linearer Ansatz im Parallelogramm
                //
//                out_fUVValue=in_fAlpha[0] + in_fAlpha[1]*in_fXi + in_fAlpha[2]*in_fEta + in_fAlpha[3]*in_fXi*in_fEta;

				double a1=in_fAlpha[0]*(1.0-in_fXi)+in_fAlpha[1]*(in_fXi);
				double a2=in_fAlpha[3]*(1.0-in_fXi)+in_fAlpha[2]*(in_fXi);

				out_fUVValue=a1*(1.0-in_fEta)+a2*(in_fEta);

                break;
            }
		case eET_TRIANGLE2:
        case eET_PARALLELOGRAMM2:
			{
				//-- Koordinaten-Transformation fr krummliniges Dreieck
                //
				int n=m_eType;

				// fuck, hier noch hoffnungslos unoptimiert
				CMatrixNMArray<double> A(n,n,m_eType==eET_TRIANGLE2?A_tri_krum:A_rect_krum);

				CVectorNArray<double> geg(n);
				CVectorNArray<double> ges(n);

				for(int i=0;i<n;i++) geg.P(i,in_vXYCoord[i].x);

				ges=A*geg;

	
				if(m_eType==eET_TRIANGLE2)
					out_fXValue= ges.P(0)
								+ges.P(1)*in_fXi
								+ges.P(2)*in_fEta
								+ges.P(3)*in_fXi*in_fXi
								+ges.P(4)*in_fXi*in_fEta
								+ges.P(5)*in_fEta*in_fEta;
				  else
					out_fXValue= ges.P(0)
								+ges.P(1)*in_fXi
								+ges.P(2)*in_fEta
								+ges.P(3)*in_fXi*in_fXi
								+ges.P(4)*in_fXi*in_fEta
								+ges.P(5)*in_fEta*in_fEta
								+ges.P(6)*in_fXi*in_fXi*in_fEta
								+ges.P(7)*in_fXi*in_fEta*in_fEta;

				for(i=0;i<n;i++) geg.P(i,in_vXYCoord[i].y);

				ges=A*geg;

				if(m_eType==eET_TRIANGLE2)
					out_fYValue= ges.P(0)
								+ges.P(1)*in_fXi
								+ges.P(2)*in_fEta
								+ges.P(3)*in_fXi*in_fXi
								+ges.P(4)*in_fXi*in_fEta
								+ges.P(5)*in_fEta*in_fEta;
				  else
					out_fYValue= ges.P(0)
								+ges.P(1)*in_fXi
								+ges.P(2)*in_fEta
								+ges.P(3)*in_fXi*in_fXi
								+ges.P(4)*in_fXi*in_fEta
								+ges.P(5)*in_fEta*in_fEta
								+ges.P(6)*in_fXi*in_fXi*in_fEta
								+ges.P(7)*in_fXi*in_fEta*in_fEta;

				for(i=0;i<n;i++) geg.P(i,in_fAlpha[i]);

				ges=A*geg;

				if(m_eType==eET_TRIANGLE2)
					out_fUVValue= ges.P(0)
								+ges.P(1)*in_fXi
								+ges.P(2)*in_fEta
								+ges.P(3)*in_fXi*in_fXi
								+ges.P(4)*in_fXi*in_fEta
								+ges.P(5)*in_fEta*in_fEta;
				  else
					out_fUVValue= ges.P(0)
								+ges.P(1)*in_fXi
								+ges.P(2)*in_fEta
								+ges.P(3)*in_fXi*in_fXi
								+ges.P(4)*in_fXi*in_fEta
								+ges.P(5)*in_fEta*in_fEta
								+ges.P(6)*in_fXi*in_fXi*in_fEta
								+ges.P(7)*in_fXi*in_fEta*in_fEta;
				break;
			}
        }
    }

private:
    //! Copy-Konstruktor als private, da keine Kopien erlaubt werden sollen. Call by Reference.
    CFemElement(const CFemElement&);
    //! Zuweisungsoperator als private, um Kopieren zu verhindern
    CFemElement& operator=(const CFemElement&);

    //! Setzt interne Attribute, die vom Typ des Elements abhngen und
    //! nur einmal bestimmt werden mssen.
    void SetHelperAttributes()
    {
        //-- Anzahl der Kanten bestimmen + den Abstand zum nchsten Endpunkt der Kante
        //
        switch (m_eType)
        {
        case eET_TRIANGLE:
            {
                //-- drei Kanten ohne Zwischenpunkte
                //
                m_iEdgeCount=3;
                break;
            };
        case eET_TRIANGLE2:
            {
                //-- drei Kanten mit je einem Zwischenpunkt
                //
                m_iEdgeCount=3;
                break;
            };

        case eET_PARALLELOGRAMM:
            {
                //-- vier Kanten ohne Zwischenpunkte
                //
                m_iEdgeCount=4;
                break;
            }

        case eET_PARALLELOGRAMM2:
            {
                //-- vier Kanten mit je einem Zwischenpunkt
                //
                m_iEdgeCount=4;
                break;
            }

        case eET_UNKNOWN    : //Keine Kanten vorhanden
        default             : m_iEdgeCount=0;
        }
    }



};

#endif //_DVA_FEM_ELEMENT_H__