//-------------------------------------------------------------------+
// FemGeoEdge.h                                                      |
//-------------------------------------------------------------------+
// Definiton of CGeoFemEdge. Represents edges from geometrical       |
//                           elements                                |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 22.02.2001
#ifndef _DVA_FEM_GEO_EDGE_H__
#define _DVA_FEM_GEO_EDGE_H__

/*!
    \brief Klasse zur Reprsentation von Kanten zwischen zwei geometrischen Elementen
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
class CFemGeoEdge
{
private:
    int m_iStartNodeNr;     //!< Anfangsknotenpunkt
    int m_iEndNodeNr;       //!< Endknotenpunkt
    int m_iNewNodeNr;       //!< Neuer Zwischenpunkt bei einem Halbierungsverfahren, sonst 0

    int m_iLeftElement;     //!< Nullindizierter Index des links angrenzenden Elements, -1 => Kante=Rand
    int m_iRightElement;    //!< Nullindizierter Index des rechts angrenzenden Elements, -1 => Kante=Rand

    float m_fSquareLength;  //!< L&auml;nge der Kante im Quadrat

    //-- kein Kopieren erlauben
    CFemGeoEdge(const CFemGeoEdge&);
    CFemGeoEdge& operator=(const CFemGeoEdge&);

public:
    //! Defaultkonstruktor setzt alle Elemente auf Null
    CFemGeoEdge():
      m_iStartNodeNr(0),
      m_iEndNodeNr(0),
      m_iNewNodeNr(0),
      m_iLeftElement(-1),
      m_iRightElement(-1),
      m_fSquareLength(0.0f)
    {}
      CFemGeoEdge(const int in_iStartNodeNr, const int in_iEndNodeNr):
      m_iStartNodeNr(in_iStartNodeNr),
      m_iEndNodeNr(in_iEndNodeNr),
      m_iNewNodeNr(0),
      m_iLeftElement(-1),
      m_iRightElement(-1),
      m_fSquareLength(0.0f)
    {}

    //! Vergleichsoperator, zwei Kanten sind gleich genau, dann wenn Start- und Endpunkt gleich sind
    //! \param rhs rechte Seite des Operators
    //! \return true, falls die Kanten gleich sind, sonst false
    bool operator==(const CFemGeoEdge& rhs) const
    {
        return (m_iStartNodeNr==rhs.m_iStartNodeNr) && (m_iEndNodeNr==rhs.m_iEndNodeNr);
    }

    //! Vergleicht, eine Kante gegeben durch Start- und Endpunkt auf Gleichheit ab wobei die Orientierung &uuml;bereinstimmen muss.
    //! \param in_iStartNodeNr Startpunkt der Kante
    //! \param in_iEndNodeNr Endpunkt der Kante
    //! \return true, falls die beiden Punkte auf der Kante liegen, sonst false
    bool CompareStrong(const int in_iStartNodeNr, const int in_iEndNodeNr) const
    {
        return (m_iStartNodeNr==in_iStartNodeNr) && (m_iEndNodeNr  ==in_iEndNodeNr);
    }

    //! Vergleicht, eine Kante gegeben durch Start- und Endpunkt auf Gleichheit ab. Orientierung der Kanten ist irrelevant.
    //! \param in_iStartNodeNr Startpunkt der Kante
    //! \param in_iEndNodeNr Endpunkt der Kante
    //! \return true, falls die beiden Punkte auf der Kante liegen, sonst false
    bool Compare(const int in_iStartNodeNr, const int in_iEndNodeNr) const
    {
        return (
                ((m_iStartNodeNr==in_iStartNodeNr) && (m_iEndNodeNr  ==in_iEndNodeNr)) ||
                ((m_iEndNodeNr  ==in_iStartNodeNr) && (m_iStartNodeNr==in_iEndNodeNr))
               );
    }

    //! Liefert den Startpunkt der Kante
    //! \return Knotenpunktnummer des Startpunktes
    int GetStartNodeNr() const { return m_iStartNodeNr; }

    //! Setzt den Startpunkt der Kante
    //! \param in_iStartNodeNr
    void SetStartNodeNr(const int in_iStartNodeNr) { m_iStartNodeNr=in_iStartNodeNr; }

    //! Liefert den Endpunkt der Kante
    //! \return Knotenpunktnummer des Endpunktes
    int GetEndNodeNr() const { return m_iEndNodeNr; }

    //! Setzt den Endpunkt der Kante
    //! \param in_iEndNodeNr Knotenpunktnummer des Endpunktes
    void SetEndNodeNr(const int in_iEndNodeNr) { m_iEndNodeNr=in_iEndNodeNr; }

    //! Liefert den Zwischenpunkt der Kante
    //! \return Knotenpunktnummer des Zwischenpunktes
    int GetNewNodeNr() const { return m_iNewNodeNr; }

    //! Setzt den Zwischenpunkt der Kante
    //! \param in_iEndNodeNr Knotenpunktnummer des Zwischenpunktes
    void SetNewNodeNr(const int in_iNewNodeNr) { m_iNewNodeNr=in_iNewNodeNr; }

    //! Liefert die Elementnummer des links angrenzenden Elements
    //! \return Nullindizierte Elementnummer
    int GetLeftElement() const { return m_iLeftElement; }

    //! Setzt die Elementnummer des links angrenzenden Elements
    //! \param in_iLeftElement Nullindizierte Elementnummer
    void SetLeftElement(const int in_iLeftElement) { m_iLeftElement=in_iLeftElement; }

    //! Liefert die Elementnummer des rechts angrenzenden Elements
    //! \return Nullindizierte Elementnummer
    int GetRightElement() const { return m_iRightElement; }

    //! Setzt die Elementnummer des rechts angrenzenden Elements
    //! \param in_iRightElement Nullindizierte Elementnummer
    void SetRightElement(const int in_iRightElement) { m_iRightElement=in_iRightElement; }

    //! Liefert die L&auml;nge der Kante im Quadrat
    //! \return L&auml;nge der Kante
    float GetSquareLength() const { return m_fSquareLength; }

    //! Setzt die L&auml;nge im Quadrat der Kante auf den &uuml;bergebenen Wert
    //! \param in_fSquareLength L&auml;nge der Kante im Quadrat
    void SetSquareLength(const float in_fSquareLength) { m_fSquareLength=in_fSquareLength; }
};
#endif//_DVA_FEM_GEO_EDGE_H__