#include "TokenParser.h"

#include <assert.h>

//-- Maximale Gre des LineBuffers bei Benutzung von getline(..)
//
const int g_iMAX_LB=2024;

//! Defaultkonstruktor
CTokenParser::CTokenParser():
    m_iLineCount(0),
    m_bReadNewLine(true)
{}


//! Destruktor
CTokenParser::~CTokenParser()
{}


//! &Ouml;ffnet eine Textdatei zum Parsen
bool CTokenParser::LoadFile(const char *in_pFileName)
{
    assert(in_pFileName != NULL);
    if (in_pFileName == NULL)
        return false;

    if (m_ifsSourceFile.is_open())
        m_ifsSourceFile.close();

    m_ifsSourceFile.open(in_pFileName);
    if (m_ifsSourceFile.is_open())
    {
        m_iLineCount=0;
        m_bReadNewLine=true;
        return true;
    }

    return false;
}


/*!
    Pr&uuml;ft ob ein bestimmtes Token vorhanden ist, und setzt die Leseposition weiter.
*/
//! \param in_pToken Tokenbezeichner
//! \param in_bCaseSensitive Mit diesem Flag kann bestimmt werden, ob die Gro- und Kleinschreibung des Tokens relevant ist.
//! \param out_iLineNr Liefert die Zeilennummer, in der das Token gefunden wurde
//! \return true bei Erfolg, false sonst
bool CTokenParser::GetToken(const char* in_pToken, const bool in_bCaseSensitive, int &out_iLineNr)
{
    if (m_bReadNewLine)
    {
        GetNewLine();
        out_iLineNr=m_iLineCount;
        m_sCurrentToken=GetTokenName(m_sCurrentLine);
    }
    else
        m_bReadNewLine=true;

    if (in_bCaseSensitive)
    if (m_sCurrentToken.find(in_pToken) != string::npos)
        return true;

    if (stricmp(in_pToken, m_sCurrentToken.c_str())==0)
        return true;

    return false;
}


//! Extrahiert aus dem Tokenstrom einen Integerwert.
//! \param out_iValue Der gefunde Wert.
//! \param out_iLineNr Liefert die Zeilennummer in der sich der Wert befindet.
//! \return true bei Erfolg, false sonst
bool CTokenParser::GetIntegerValue(int &out_iValue, int &out_iLineNr)
{
    GetNewLine();

    out_iLineNr=m_iLineCount;
    m_sCurrentToken=GetTokenName(m_sCurrentLine);

    if (m_sCurrentToken.empty())
        return false;
    if (isalpha(m_sCurrentToken[0]))
        return false;

    out_iValue=atoi(m_sCurrentToken.c_str());
    return true;
}

//! Extrahiert aus dem Tokenstrom einen Floatwert.
//! \param out_iValue Der gefunde Wert.
//! \param out_iLineNr Liefert die Zeilennummer in der sich der Wert befindet.
//! \return true bei Erfolg, false sonst
bool CTokenParser::GetFloatValue(float &out_fValue, int &out_iLineNr)
{
    GetNewLine();

    out_iLineNr=m_iLineCount;
    m_sCurrentToken=GetTokenName(m_sCurrentLine);

    if (m_sCurrentToken.empty())
        return false;
    if (isalpha(m_sCurrentToken[0]))
        return false;

    out_fValue=atof(m_sCurrentToken.c_str());
    return true;
}

//! Extrahiert aus dem Tokenstrom den n&auml;chst folgenden zusammenh&auml;ngenden String.<br>
//! Dieser String wird als Optional betrachtet, d.h. es wird keine neue Zeile geladen bzw.<br>
//! muss der String in der aktuellen Zeile vorliegen.
//! \param out_iLineNr Liefert die Zeilennummer in der sich der Wert befindet.
//! \return String falls vorhanden, sonst NULL
const char* CTokenParser::GetOptionalStringValue(int &out_iLineNr)
{
    out_iLineNr=m_iLineCount;
    m_sCurrentToken=GetTokenName(m_sCurrentLine);

    if (m_sCurrentToken.empty())
        return NULL;

    return m_sCurrentToken.c_str();
}

//! Legt das Token f&uuml;r einzeilige Kommentare fest
//! \param in_pCommentToken Symbolfolge, die einen einzeiligen Kommentar einleitet
void CTokenParser::SetLineCommentToken(const char* in_pCommentToken)
{
    assert(in_pCommentToken != NULL);
    m_sCommentToken=in_pCommentToken;
}

//-- Liefert den Text zwischen zwei <>-Klammern-Paaren in outText, und entfernt diesen aus inBuffer
//   Rckgabewert: entfernter HTML-Tag
//
string CTokenParser::GetTokenName(string& in_strBuffer)
{
    TrimLeft(in_strBuffer);

    //-- Lnge des Tokens feststellen
    //
    int i=0;
    while(in_strBuffer[i] != '\0')
    {
        //-- Solange die Schleife durchlaufen bis eins der folgenden Trennzeichen
        //   zwischen zwei Tokens gefunden wurde
        //
	    if (
            (in_strBuffer[i] == ' ')  ||    //Space
            (in_strBuffer[i] == '\n') ||    //Newline
            (in_strBuffer[i] == 0xA)  ||    //Windows-Newline
            (in_strBuffer[i] == 0xD)        //Windows-Newline
           )
        {
            break;
        }
        i++;
    }

    //-- Token extrahieren
    //
    string strTokenName=in_strBuffer.substr(0, i);

    //-- Eingabebuffer um Token verkrzen
    //
    in_strBuffer=in_strBuffer.substr(i);

    //-- Im Reststring nochmals auf WhiteSpace-Zeichen prfen
    TrimLeft(in_strBuffer);

    return strTokenName;
}

//! Schneidet von links her alle Whitespace-Zeichen ab
//! \param in_strBuffer Zu bearbeitender Eingabestring
void CTokenParser::TrimLeft(string& in_strBuffer)
{
    //-- Durchlaufe die Schleife solange, bis kein WhiteSpace-Zeichen folgt
    //
    int i=0;
    while(in_strBuffer[i] != '\0')
    {
	    if (
            (in_strBuffer[i] == ' ')  ||    //Space
            (in_strBuffer[i] == '\n') ||    //Newline
            (in_strBuffer[i] == 0xA)  ||    //Windows-Newline
            (in_strBuffer[i] == 0xD)        //Windows-Newline
           )
        {
            i++;
            continue;
        }
        break;
    }

    //-- WhiteSpaces abschneiden
    //
    in_strBuffer=in_strBuffer.substr(i);
}

//! L&auml;dt eine komplette Zeile aus der Quelldatei und &uberspringt dabei einzeilige Kommentare, 
//! falls ein Token daf&uuml;r definiert ist.
void CTokenParser::GetNewLine()
{
    //-- Nur wenn die aktuelle Zeile bereits abgearbeitet ist, muss eine neue geholt werden.
    //
    if (m_sCurrentLine.empty() == false)
        return;

    while(true)
    {
        //-- Nchste Zeile laden
        //
        char pBuffer[g_iMAX_LB];
        m_ifsSourceFile.getline(pBuffer, g_iMAX_LB);
        m_sCurrentLine=pBuffer;

        m_iLineCount++;

        //-- Leerzeilen werden bersprungen
        //
        if (m_sCurrentLine.empty())
            continue;

        //-- Falls ein Token fr einzeilige Kommentare definiert ist
        //   wird berprft, ob es sich um die zuvor eingelesen Zeile um einen Kommentar handelt.
        if (!m_sCommentToken.empty())
        {
            //-- Falls ein Kommentar vorhanden ist, wird die nchste Zeile geladen
            if (m_sCurrentLine.find(m_sCommentToken) != string::npos)
                continue;
        }
        break;
    }
}

