//-------------------------------------------------------------------+
// TokenParser.h                                                     |
//-------------------------------------------------------------------+
// Definiton of CTokenParser. Helper-Class for Parsing Textfiles.    |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 31.01.2001
#ifndef _TOKEN_PARSER_H__
#define _TOKEN_PARSER_H__

#include <assert.h>
#include <string>
#include <fstream>
using namespace std;


/*!
    \brief Hilfsklasse zum zeilenweise Parsen von Textdateien
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
class CTokenParser
{
private:
    int      m_iLineCount;      //!< Aktuelle Zeile der Leseposition innerhalb der Textdatei
    ifstream m_ifsSourceFile;   //!< Textdatei, von der gelesen wird
    string   m_sCurrentToken;   //!< aktuell gescannter Tokenbezeichner, kann zur Fehlerausgabe benutzt werden
    string   m_sCurrentLine;    //!< komplette Zeile, die derzeit gescannt wird
    string   m_sCommentToken;   //!< Symbol, Symbolfolge, die einen einzeiligen Kommentar einleiten

    bool     m_bReadNewLine;    //!< Flag, ob GetNewLine() benutzt werden soll. Kann dazu benutzt werden um den Parser auf dem aktuellen Token zu belassen.

private:
    //! Extrahiert aus dem Eingabestring ein zusammenh&auml;ngendes Wort
    //! \param in_strBuffer Eingabestring, der um das gefundene Token gekrzt wird
    //! \return Token oder leerer String
    string GetTokenName(string& in_strBuffer);

    //! Schneidet von links her alle Whitespace-Zeichen ab
    //! \param in_strBuffer Zu bearbeitender Eingabestring
    void TrimLeft(string& in_strBuffer);

    //! L&auml;dt eine komplette Zeile aus der Quelldatei und &uberspringt dabei einzeilige Kommentare, 
    //! falls ein Token daf&uuml;r definiert ist.
    void GetNewLine();

public:
    //! Defaultkonstruktor
    CTokenParser();

    //! Konstruktor mit Angabe einer Symbolfolge f&uuml;r einzeilige Kommentare
    //! \param in_pCommentToken Symbolfolge, die einen einzeiligen Kommentar einleitet
    CTokenParser(const char* in_pCommentToken)
    {
        assert(in_pCommentToken != NULL);
        m_sCommentToken=in_pCommentToken;
    }

    //! Destruktor
    ~CTokenParser();

    //! &Ouml;ffnet eine Textdatei zum parsen
    bool LoadFile(const char* in_pFileName);

    //! Pr&uuml;ft ob ein bestimmtes Token vorhanden ist, und setzt die Leseposition weiter.
    //! \param in_pToken Tokenbezeichner
    //! \param in_bCaseSensitive Mit diesem Flag kann bestimmt werden, ob die Gro- und Kleinschreibung des Tokens relevant ist.
    //! \param out_iLineNr Liefert die Zeilennummer, in der das Token gefunden wurde
    //! \return true bei Erfolg, false sonst
    bool GetToken(const char* in_pToken, const bool in_bCaseSensitive, int &out_iLineNr);

    //! Extrahiert aus dem Tokenstrom einen Integerwert.
    //! \param out_iValue Der gefunde Wert.
    //! \param out_iLineNr Liefert die Zeilennummer in der sich der Wert befindet.
    //! \return true bei Erfolg, false sonst
    bool GetIntegerValue(int &out_iValue, int &out_iLineNr);

    //! Extrahiert aus dem Tokenstrom einen Floatwert.
    //! \param out_iValue Der gefunde Wert.
    //! \param out_iLineNr Liefert die Zeilennummer in der sich der Wert befindet.
    //! \return true bei Erfolg, false sonst
    bool GetFloatValue(float &out_fValue, int &out_iLineNr);

    //! Extrahiert aus dem Tokenstrom den n&auml;chst folgenden zusammenh&auml;ngenden String.<br>
    //! Dieser String wird als Optional betrachtet, d.h. es wird keine neue Zeile geladen bzw.<br>
    //! muss der String in der aktuellen Zeile vorliegen.
    //! \param out_iLineNr Liefert die Zeilennummer in der sich der Wert befindet.
    //! \return String falls vorhanden, sonst NULL
    const char* GetOptionalStringValue(int &out_iLineNr);

    //! Legt das Token f&uuml;r einzeilige Kommentare fest
    //! \param in_pCommentToken Symbolfolge, die einen einzeiligen Kommentar einleitet
    void SetLineCommentToken(const char* in_pCommentToken);

    //! Liefert das Token auf dem sich die Leseposition befindet (LookAheadSymbol)
    //! \return LookAheadSymbol
    const char* GetCurrentToken() const { return m_sCurrentToken.c_str(); }

    //! Vergleich den Eingabestring mit dem LookAheadSymbol (case-sensitiv)
    //! \param in_pString Eingabestring der mit dem aktuellen Token verglichen wird
    //! \return true falls sie identisch sind, sonst false
    bool CompareCurrentToken(const char* in_pString) { return strcmp(in_pString, m_sCurrentToken.c_str())==0; }

    //! Bei einem nchsten GetToken()-Aufruf, wird kein neues Token von der Datei geladen sondern, das aktuelle Token benutzt.
    //! Dies kann z. B. nach einem fehlgeschlagenen GetFloat/GetInteger-Aufruf benutzt werden.
    void SetBack() { m_bReadNewLine=false; }
};

#endif //_TOKEN_PARSER_H__