//-------------------------------------------------------------------+
// stl_vector.h                                                      |
//-------------------------------------------------------------------+
// Nachbau der stl-vector-klasse                                     |
//-------------------------------------------------------------------+
// Author: Tobias Lang                                               |
//-------------------------------------------------------------------+
// (c) Copyright 2001 by Tobias Lang                                 |
//-------------------------------------------------------------------+ t.l. 01.03.2001
#ifndef _STL_VECTOR_H__
#define _STL_VECTOR_H__

/*!
    \brief Kontainer-Klasse die einen Vektor linear mit beliebigen Elementen h&auml;lt.
    \author Tobias Lang
    \version 1.0
    \date    2001
*/
template<class T>
class stl_vector
{
private:
    T*  m_pData;            //!< Statisches Array der Daten
    int m_iElements;        //!< aktuelle Anzahl an belegten Elementen im Array
    int m_iReservedSize;    //!< Gr&ouml;sse des Arrays

    //-- kein Kopieren erlauben
    stl_vector(const stl_vector&);
    stl_vector& operator=(const stl_vector&);

public:
    //! Defaultkonstruktur setzt alle Elemente auf Null
    stl_vector():
      m_pData(NULL),
      m_iElements(0),
      m_iReservedSize(0)
    {}

    //! Destruktor gibt den allokierten Speicher wieder frei
    ~stl_vector()
    {
        delete [] m_pData;
    }

    //! Schreibender Zugriffsoperator. Sollte nur nach einem reserve benutzt werden. Es findet keine Index-&Uuml;berpr&uuml;fung statt!
    //! \param in_iIndex Nullindizierter Index auf das Element.
    //! \return Indizierte Element.
    T& operator[](const int in_iIndex)
    {
        return m_pData[in_iIndex];
    }

    //! Gesicherter Schreibzugriff auf den Vektor. Etwas langsamer als der operator[].
    //! \param in_iIndex Nullindizierter Index auf das Element.
    //! \param in_TElement Element das gesetzt werden soll.
    //! \return true bei Erfolg, sonst false
    bool Set(const int in_iIndex, const T& in_TElement)
    {
        if ((in_iIndex >=0) && (in_iIndex<m_iElements))
        {
            m_pData[in_iIndex]=in_TElement;
            return true;
        }
        return false;
    }

    //! Lesender Zugriff auf den Vektor. Es findet keine Index-&Uuml;berpr&uuml;fung statt!
    //! \param in_iIndex Nullindizierter Index auf das Element.
    //! \return Indizierte Element.
    const T& operator[](const int in_iIndex) const
    {
        return m_pData[in_iIndex];
    }

    //! L&ouml;scht alle Elemente.
    void clear()
    {
        delete [] m_pData;
        m_pData=NULL;
        m_iElements=m_iReservedSize=0;
    }

    //! Liefert die Anzahl der vorhandenen Elemente im Vektor.
    //! \return Anzahl der Elemente.
    int size() const { return m_iElements; }

    //! Liefert die Gr&ouml;sse des reservierten Speichers f&uuml;r den Vektor.
    //! \return Reservierte Gr&ouml;sse des Vektors.
    int capacity() const { return m_iReservedSize; }

    //! F&uuml;gt am Ende des Vektors ein neues Element ein. Reicht der Speicher nicht aus, wird
    //! die aktuelle Gr&ouml;sse verdoppelt und die alten Elemente kopiert. Die Anzahl der Elemente wird um
    //! ein Element erh&ouml;ht.
    //! \param in_TElement Element, dass eingef&uuml;gt wird.
    void push_back(const T& in_TElement)
    {
        if (m_iReservedSize<=m_iElements)
        {
            //-- Realloc
            //
            m_iReservedSize*=2;
            T* temp=new T[m_iReservedSize];
            assert(temp != NULL);

            memcpy(temp, m_pData, m_iElements*sizeof(T));

            delete [] m_pData;
            m_pData=temp;
        }
        m_pData[m_iElements]=in_TElement;
        m_iElements++;
    }

    //! Reserviert f&uuml;r eine gewisse Gr&ouml;sse Speicher. Die alten Elemente werden &uuml;bernommen.
    //! \param in_iSize Speichergr&ouml;sse.
    void reserve(const int in_iSize)
    {
        //-- Realloc
        //
        m_iReservedSize=in_iSize;
        T* temp=new T[m_iReservedSize];
        assert(temp != NULL);

        if (m_iElements>m_iReservedSize)
            m_iElements=m_iReservedSize;

        memcpy(temp, m_pData, m_iElements*sizeof(T));

        delete [] m_pData;
        m_pData=temp;
    }

    //! F&uuml;gt hinter dem Index ein neues Element ein. Reicht der Speicher nicht aus, wird
    //! die aktuelle Gr&ouml;sse verdoppelt und die alten Elemente kopiert. Die Anzahl der Elemente wird um
    //! ein Element erh&ouml;ht.
    //! \param in_iIndex Nullindizierter Index auf das Element das vor dem Einzuf&uuml;genden liegt.
    //! \param in_TElement Neue Element.
    void insert(const int in_iIndex, const T& in_TElement)
    {
        assert(in_iIndex < m_iElements);

        if (m_iReservedSize<=m_iElements)
        {
            //-- Realloc
            //
            m_iReservedSize*=2;
            T* temp=new T[m_iReservedSize];
            assert(temp != NULL);

            memcpy(temp, m_pData, m_iElements*sizeof(T));

            delete [] m_pData;
            m_pData=temp;
        }

        //-- Anzahl der Elemente, die nach dem Einzufgenden folgen.
        //
        int iTailSize=m_iElements-in_iIndex-1;

        //-- Alle Elemente um eine Position nach rechts schieben.
        //
        for (int i=0; i<iTailSize; i++)
            m_pData[m_iElements-i]=m_pData[m_iElements-i-1];

        //-- Neues Element einfgen
        //
        m_pData[in_iIndex+1]=in_TElement;
        m_iElements++;
    }
};

#endif//_STL_VECTOR_H__